package com.tinysine.tosrwifi;

import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.HorizontalScrollView;
import android.widget.ScrollView;
import android.widget.TextView;

import com.tinysine.tosrwifi.io.Util;
import com.tinysine.tosrwifi.service.SocketService;

public class SettingConfig extends Activity {

	public static String address = "192.168.1.1";
	public static int port = 2000;
	private static ScrollView scrollView;

	private String name;
	private String wifiName;
	private String password;
	private String ipString;

	private TextView textView;
	private SocketService mService;

	private String[] sendCmds = new String[10];
	private int cmdPos = 0;

	private SharedPreferences preferences;

	public static final String PREFERENCE_CONNECT_NAME = "connectName";
	public static final String PREFERENCE_WIFI_SSID = "WiFiName";
	public static final String PREFERENCE_WIFI_PASSWORD = "WiFiPassword";
	public static final String PREFERENCE_WIFI_IP = "WiFiIP";

	private Button finish;
	private Button retry;
	private ArrayList<String> devices = new ArrayList<String>();

	private SendThread sendThread = new SendThread();

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		// TODO Auto-generated method stub
		super.onCreate(savedInstanceState);

		setContentView(R.layout.setting_config);
		preferences = getSharedPreferences(Bluemd.PREFS_NAME, 0);
		mService = new SocketService(mHandler, this);
		Intent intent = getIntent();
		name = intent.getExtras().getString("name");
		wifiName = intent.getExtras().getString("WiFiName");
		password = intent.getExtras().getString("WiFiPassword");
		ipString = intent.getExtras().getString("IP");
		scrollView = (ScrollView) findViewById(R.id.scrollView);
		textView = (TextView) findViewById(R.id.textView);
		finish = (Button) findViewById(R.id.Finish);
		retry = (Button) findViewById(R.id.Retry);
		finish.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				String deviceString = preferences.getString("devices", "");
				String[] tmp = null;
				if (deviceString != null && deviceString.length() > 0
						&& !deviceString.equals("")) {
					tmp = deviceString.split(",");
					if (tmp != null) {
						for (String tmpString : tmp)
							devices.add(tmpString);
					}
				}
				devices.add(name + " " + ipString + ":2000");
				StringBuilder sb = new StringBuilder();
				for (String tmpTmp : devices) {
					sb.append(tmpTmp);
					sb.append(",");
				}
				String resString = sb.toString().substring(0,
						sb.toString().length());

				Editor editor = preferences.edit();
				editor.putString("devices", resString);
				editor.commit();

				Intent intent = new Intent();
				intent.putExtra(PREFERENCE_CONNECT_NAME, name);
				intent.putExtra(PREFERENCE_WIFI_SSID, wifiName);
				intent.putExtra(PREFERENCE_WIFI_PASSWORD, password);
				intent.putExtra(PREFERENCE_WIFI_IP, ipString);
				setResult(RESULT_OK);
				SharedPreferences userInfo = getSharedPreferences("user_info",
						0);
				Editor editorEditor = userInfo.edit();
				editorEditor.putString("ip", ipString);
				editorEditor.putInt("port", 2000);
				editorEditor.putString("name", name);
				editorEditor.commit();
				SettingConfig.this.finish();
			}
		});
		retry.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				textView.append("Waiting for service.Please try later...\n");
				textView.append("Retry:\n");
			}
		});
		sendCmds[0] = "$$$";
		sendCmds[1] = "set wlan ssid " + replaceSpace(wifiName) + "\r\r\n";
		sendCmds[2] = "set wlan phrase " + password + "\r\r\n";
		sendCmds[3] = "set wlan join 1\r\r\n";
		sendCmds[4] = "set ip a " + ipString + "\r\r\n";
		sendCmds[5] = "set ip dhcp 0\r\r\n";
		sendCmds[6] = "set sys printlvl 0\r\r\n";
		sendCmds[7] = "set comm idle 60\r\r\n";
		sendCmds[8] = "save\r\r\n";
		sendCmds[9] = "reboot\r\r\n";

		startConnect();

	}

	private String getByteString(byte[] dats, int len) {
		Charset cs = Charset.forName("utf-8");
		ByteBuffer bb = ByteBuffer.allocate(dats.length);
		bb.put(dats);
		bb.flip();
		CharBuffer cBuffer = cs.decode(bb);
		return new String(cBuffer.array());
	}

	private class SendThread extends Thread {
		boolean sendFlag = false;

		@Override
		public void run() {
			while (true) {
				if (sendFlag) {
					try {
						sendString(sendCmds[cmdPos]);
						mHandler.obtainMessage(Util.STATE_CONFIG,
								"Send:" + sendCmds[cmdPos]).sendToTarget();
						cmdPos++;
						if (cmdPos >= sendCmds.length) {
							cmdPos = 0;
							sendFlag = false;
							mHandler.sendMessage(Message
									.obtain(mHandler,
											Util.STATE_CONFIG,
											"Configuration is complete, Please connect your router. \nSwitch your relay board to normal mode and restart this APP"));
						}
					} catch (Exception e) {
						e.printStackTrace();
					}

					try {
						Thread.sleep(2000);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}

				} else {
					try {
						Thread.sleep(500);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}
		}

		public void Restart() {
			sendFlag = true;
			cmdPos = 0;
			start();
		}

		public void Stop() {
			sendFlag = false;
		}
	}

	public void sendString(String data) {
		mService.sendBytes(data.getBytes());
	}

	public void startConnect() {
		Log.e("SettingConfig", mService == null ? "ServiceConn:NULL" : "Not");
		if (!mService.isConnected()) {
			cmdPos = 0;
			mService.connectService(address, port);
		} else if (mService.isConnected()) {
			try {
				mService.disConnect();
				mService.connectService(address, port);
				cmdPos = 0;
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}
	}

	public static String replaceSpace(String value) {
		Pattern pattern = Pattern.compile("\\s+");
		Matcher matcher = pattern.matcher(value);
		if (matcher.find()) {
			return matcher.replaceAll("\\$");
		}
		return value;
	}

	private Handler mHandler = new Handler(new Handler.Callback() {

		@Override
		public boolean handleMessage(Message msg) {
			// TODO Auto-generated method stub
			switch (msg.what) {
			case Util.WIFI_STATE: {
				switch (msg.arg1) {
				case Util.STATE_CONNECTED: {
					textView.append("Connected:" + address + ":" + port + "\n");
					scrollView.fullScroll(HorizontalScrollView.FOCUS_DOWN);
					sendThread.Restart();
				}
					break;
				case Util.STATE_NONE: {
					textView.append("Not connected:" + address + ":" + port
							+ "\n");
					scrollView.fullScroll(HorizontalScrollView.FOCUS_DOWN);
					sendThread.Stop();
				}
					break;
				case Util.STATE_CONNECTING: {
					textView.append("Connecting:" + address + ":" + port + "\n");
					scrollView.fullScroll(HorizontalScrollView.FOCUS_DOWN);
					sendThread.Stop();
				}
					break;

				default:
					break;
				}
			}
				break;
			case Util.WIFI_DATA: {
				byte[] datas = (byte[]) msg.obj;
				if (datas != null) {
					textView.append("\nReceive:"
							+ getByteString((byte[]) msg.obj, msg.arg2));
					scrollView.fullScroll(HorizontalScrollView.FOCUS_DOWN);
				}
			}
				break;
			case Util.STATE_CONFIG: {
				textView.append("\n" + msg.obj);
				scrollView.fullScroll(HorizontalScrollView.FOCUS_DOWN);
			}
			default:
				break;
			}
			return false;
		}
	});
}